unit UnidadProcedimientos;

interface

uses classes, dbierrs, dbtables, SysUtils, Windows, Messages, Variants, Controls,
  Dialogs, ExtCtrls,  inifiles, registry, shlobj;

  function bdeInstalado () : boolean;
  function ejecutadaYa (nombreAplicacion : string) : boolean;
  function pMes (mes : integer) : string;
  function pFecha (dia, mes, ano : word) : tdate;
  function reemplazarTexto (cadena, textoAReemplazar, textoNuevo : string) : string;
  function Redondear (Numero : Double; PrecisionDec : byte) : double;
  function leBoolINI (clave, cadena : string; defecto : boolean) : boolean;
  function leEntINI (clave, cadena : string; defecto : integer) : integer;
  function leCadINI (clave, cadena : string; defecto : string) : string;
  procedure esBoolINI (clave, cadena : string; valor : boolean);
  procedure esEntINI (clave, cadena : string; valor : integer);
  procedure esCadINI (clave, cadena, valor : string);
  procedure esFloatINI (clave, cadena : string; valor : double);
  procedure esFechaINI (clave, cadena : string; defecto : tdatetime);
  function leFechaINI (clave, cadena : string; defecto : tdatetime) : tdatetime;
  function leFloatINI (clave, cadena : string; defecto : double) : double;
  function tipoUnidad (ruta : string) : string;
  procedure escribeRegistro (desdeClave : HKEY; ruta, cadena, valor : string;
                      crear : boolean);
  function DlgElegirDirectorio(subtitulo: string;
      nuevoEstilo: boolean) : string;
var
  vtNombreAplicacion : string;
  vtVersionAplicacion : string;

const
 BIF_NEWDIALOGSTYLE = $40;

implementation

function DlgElegirDirectorio(subtitulo: string;
   nuevoEstilo: boolean) : string;
 var
   BI: TBrowseInfo;
   IL: PItemIDList;
 begin
   Result := EmptyStr;
   with BI do begin
     lpfn:= nil;
     lParam:= 0;
     iImage:= 0;
     pidlRoot:= nil;
     hwndOwner:= GetActiveWindow;
     lpszTitle:= PChar(subtitulo);
     GetMem(pszDisplayName, MAX_PATH);
     ulFlags := BIF_RETURNONLYFSDIRS;
     if nuevoEstilo then
       ulFlags := ulFlags or BIF_NEWDIALOGSTYLE;
   end;
   IL := SHBrowseForFolder(BI);
   if SHGetPathFromIDList(IL, BI.pszDisplayName) then
     Result := IncludeTrailingPathDelimiter(
       StrPas(BI.pszDisplayName));
   FreeMem(BI.pszDisplayName);
   GlobalFreePtr(IL);
 end;


//Escribe valor en el Regedit
procedure escribeRegistro (desdeClave : HKEY; ruta, cadena, valor : string;
                      crear : boolean);
var
  reg : TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := desdeClave;
    Reg.OpenKey(ruta, crear);
    Reg.WriteString(cadena, valor);
  finally
    reg.Free;
  end;
end;

function tipoUnidad (ruta : string) : string;
Const  // Matriz con los tipos de unidades
  TiposUnidad : Array[0..6] Of String = ('Desconocido',
        'No es una unidad', 'Removible', 'Fija',
        'Remota', 'CD-ROM', 'Disco RAM');
begin
  result := TiposUnidad[GetDriveType(PChar(ruta))];
end;

function leBoolINI (clave, cadena : string; defecto : boolean) : boolean;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readbool (clave, cadena, defecto);
  finally
    free;
  end;
end;

function leEntINI (clave, cadena : string; defecto : integer) : integer;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readInteger (clave, cadena, defecto);
  finally
    free;
  end;
end;

function leCadINI (clave, cadena : string; defecto : string) : string;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readString (clave, cadena, defecto);
  finally
    free;
  end;
end;

procedure esBoolINI (clave, cadena : string; valor : boolean);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    writeBool (clave, cadena, valor);
  finally
    free;
  end;
end;

procedure esEntINI (clave, cadena : string; valor : integer);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    writeInteger (clave, cadena, valor);
  finally
    free;
  end;
end;

procedure esCadINI (clave, cadena, valor : string);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    writeString (clave, cadena, valor);
  finally
    free;
  end;
end;

procedure esFloatINI (clave, cadena : string; valor : double);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    WriteFloat (clave, cadena, valor);
  finally
    free;
  end;
end;

procedure esFechaINI (clave, cadena : string; defecto : tdatetime);
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    WriteDate (clave, cadena, defecto);
  finally
    free;
  end;
end;

function leFechaINI (clave, cadena : string; defecto : tdatetime) : tdatetime;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := ReadDate (clave, cadena, defecto);
  finally
    free;
  end;
end;

function leFloatINI (clave, cadena : string; defecto : double) : double;
begin
  with tinifile.create (changefileext(paramstr(0),'.INI')) do
  try
    result := readfloat (clave, cadena, defecto);
  finally
    free;
  end;
end;


function Redondear (Numero : Double; PrecisionDec : byte) : double;
  function Potencia(Base : Double; Exponente : byte) : Double;
  begin
    Result := Exp(Exponente * ln(Base));
  end;
begin
  result := Round(Numero * Potencia(10, PrecisionDec)) / Potencia (10, PrecisionDec);
end;

//para saber si est instaldo el BDE
function fDbiGetSysVersion(SysVerList: TStringList): SYSVersion;
var
  Month, Day, iHour, iMin, iSec: Word;
  Year: SmallInt;
begin
  Check(DbiGetSysVersion(Result));
  if (SysVerList <> nil) then
  begin
    with SysVerList do
    begin
      Clear;
      Add(Format('ENGINE VERSION=%d', [Result.iVersion]));
      Add(Format('INTERFACE LEVEL=%d', [Result.iIntfLevel]));
      Check(DbiDateDecode(Result.dateVer, Month, Day, Year));
      Add(Format('VERSION DATE=%s', [DateToStr(EncodeDate
                (Year, Month, Day))]));
      Check(DbiTimeDecode(Result.timeVer, iHour, iMin, iSec));
      Add(Format('VERSION TIME=%s', [TimeToStr(EncodeTime
                (iHour, iMin, iSec div 1000, iSec div 100))]));
    end;
  end;
end;

//Para saber si est el BDE instalado
function bdeInstalado () : boolean;
var
  hStrList : TStringList;
  Ver : SYSVersion;
begin
{  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CURRENT_USER;
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKey('SOFTWARE\Borland\Database Engine', False);
  s := '';
  try
    s := reg.ReadString('CONFIGFILE01');
  finally
    reg.CloseKey;
  end;
  if s = '' then
  begin
    bdeinstalado := false;
    exit;
  end;}
  hStrList := TStringList.Create;
  try
    Ver := fDbiGetSysVersion(hStrList);
  except
    bdeInstalado := false;
    exit;
  end;
  bdeInstalado := true;
end;


//Comprueba si la aplicacin est ejecutndose ya
function ejecutadaYa (nombreAplicacion : string) : boolean;
var
  MiMutex : Thandle;
begin
  mimutex := CreateMutex(nil, true, pchar(nombreAplicacion));
  if MiMutex = 0 then
    result := false
  else
  begin
    if GetLastError = ERROR_ALREADY_EXISTS then
      result := true
    else
      result := false;
  end;
end;


//Devuelve la fecha a partir de da, mes, ao
function pFecha (dia, mes, ano : word) : tdate;
begin
  result := EncodeDate (ano,mes,dia);
end;


//Devuelve el nombre del mes segn el nmero
function pMes (mes : integer) : string;
begin
  case mes of
    1 : result := 'Enero';
    2 : result := 'Febrero';
    3 : result := 'Marzo';
    4 : result := 'Abril';
    5 : result := 'Mayo';
    6 : result := 'Junio';
    7 : result := 'Julio';
    8 : result := 'Agosto';
    9 : result := 'Septiembre';
    10 : result := 'Octubre';
    11 : result := 'Noviembre';
    12 : result := 'Diciembre';
  end;
end;


function reemplazarTexto (cadena, textoAReemplazar, textoNuevo : string) : string;
var
  aPos : Integer;
begin
  aPos := Pos(textoAReemplazar, Cadena);
  Result := '';
  while (aPos <> 0) do begin
    Result := Result + Copy(Cadena, 1, aPos - 1) + textoNuevo;
    Delete(Cadena, 1, aPos + Length(textoAReemplazar) - 1);
    aPos := Pos(textoAReemplazar, Cadena);
  end;
  Result := Result + Cadena;
end;


end.
